@@ -262,7 +262,7 @@ class Element(Plugin):
         # Collect the composited element configuration and
         # ask the element to configure itself.
         self.__config = self.__extract_config(meta)
-        self.configure(self.__config)
+        self._configure(self.__config)
 
         # Extract Sandbox config
         self.__sandbox_config = self.__extract_sandbox_config(meta)
@@ -1085,9 +1085,12 @@ class Element(Plugin):
             # until the full cache query below.
             cached = self.__artifacts.contains(self, self.__weak_cache_key)
             if (not self.__assemble_scheduled and not self.__assemble_done and
-                    not cached and not self._pull_pending() and self._is_required()):
-                self._schedule_assemble()
-                return
+                    not cached and not self._pull_pending()):
+                # For uncached workspaced elements, assemble is required
+                # even if we only need the cache key
+                if self._is_required() or self._get_workspace():
+                    self._schedule_assemble()
+                    return
 
         if self.__strict_cache_key is None:
             dependencies = [
@@ -1107,13 +1110,17 @@ class Element(Plugin):
             self.__strong_cached = self.__artifacts.contains(self, self.__strict_cache_key)
 
         if (not self.__assemble_scheduled and not self.__assemble_done and
-                not self.__cached and not self._pull_pending() and self._is_required()):
+                not self.__cached and not self._pull_pending()):
             # Workspaced sources are considered unstable if a build is pending
             # as the build will modify the contents of the workspace.
             # Determine as early as possible if a build is pending to discard
             # unstable cache keys.
-            self._schedule_assemble()
-            return
+
+            # For uncached workspaced elements, assemble is required
+            # even if we only need the cache key
+            if self._is_required() or self._get_workspace():
+                self._schedule_assemble()
+                return
 
         if self.__cache_key is None:
             # Calculate strong cache key
@@ -1382,7 +1389,6 @@ class Element(Plugin):
     # in a subprocess.
     #
     def _schedule_assemble(self):
-        assert self._is_required()
         assert not self.__assemble_scheduled
         self.__assemble_scheduled = True
 
@@ -1390,6 +1396,8 @@ class Element(Plugin):
         for dep in self.dependencies(Scope.BUILD, recurse=False):
             dep._set_required()
 
+        self._set_required()
+
         # Invalidate workspace key as the build modifies the workspace directory
         workspace = self._get_workspace()
         if workspace:
@@ -1579,6 +1587,10 @@ class Element(Plugin):
     #   (bool): Whether a pull operation is pending
     #
     def _pull_pending(self):
+        if self._get_workspace():
+            # Workspace builds are never pushed to artifact servers
+            return False
+
         if self.__strong_cached:
             # Artifact already in local cache
             return False
