@@ -28,6 +28,18 @@ Abstract Methods
 For loading and configuration purposes, Sources must implement the
 :ref:`Plugin base class abstract methods <core_plugin_abstract_methods>`.
 
+.. attention::
+
+   In order to ensure that all configuration data is processed at
+   load time, it is important that all URLs have been processed during
+   :func:`Plugin.configure() <buildstream.plugin.Plugin.configure>`.
+
+   Source implementations *must* either call
+   :func:`Source.translate_url() <buildstream.source.Source.translate_url>` or
+   :func:`Source.mark_download_url() <buildstream.source.Source.mark_download_url>`
+   for every URL that has been specified in the configuration during
+   :func:`Plugin.configure() <buildstream.plugin.Plugin.configure>`
+
 Sources expose the following abstract methods. Unless explicitly mentioned,
 these methods are mandatory to implement.
 
@@ -149,6 +161,13 @@ class SourceFetcher():
     fetching and substituting aliases.
 
     *Since: 1.2*
+
+    .. attention::
+
+       When implementing a SourceFetcher, remember to call
+       :func:`Source.mark_download_url() <buildstream.source.Source.mark_download_url>`
+       for every URL found in the configuration data at
+       :func:`Plugin.configure() <buildstream.plugin.Plugin.configure>` time.
     """
     def __init__(self):
         self.__alias = None
@@ -171,7 +190,7 @@ class SourceFetcher():
         Implementors should raise :class:`.SourceError` if the there is some
         network error or if the source reference could not be matched.
         """
-        raise ImplError("Source fetcher '{}' does not implement fetch()".format(type(self)))
+        raise ImplError("SourceFetcher '{}' does not implement fetch()".format(type(self)))
 
     #############################################################
     #                       Public Methods                      #
@@ -184,10 +203,7 @@ class SourceFetcher():
         Args:
            url (str): The url used to download.
         """
-        # Not guaranteed to be a valid alias yet.
-        # Ensuring it's a valid alias currently happens in Project.get_alias_uris
-        alias, _ = url.split(utils._ALIAS_SEPARATOR, 1)
-        self.__alias = alias
+        self.__alias = _extract_alias(url)
 
     #############################################################
     #            Private Methods used in BuildStream            #
@@ -219,8 +235,11 @@ class Source(Plugin):
         self.__element_kind = meta.element_kind         # The kind of the element owning this source
         self.__directory = meta.directory               # Staging relative directory
         self.__consistency = Consistency.INCONSISTENT   # Cached consistency state
+
+        # The alias_override is only set on a re-instantiated Source
         self.__alias_override = alias_override          # Tuple of alias and its override to use instead
-        self.__expected_alias = None                    # A hacky way to store the first alias used
+        self.__expected_alias = None                    # The primary alias
+        self.__marked_urls = set()                      # Set of marked download URLs
 
         # FIXME: Reconstruct a MetaSource from a Source instead of storing it.
         self.__meta = meta                              # MetaSource stored so we can copy this source later.
@@ -231,7 +250,7 @@ class Source(Plugin):
         self.__config = self.__extract_config(meta)
         self.__first_pass = meta.first_pass
 
-        self.configure(self.__config)
+        self._configure(self.__config)
 
     COMMON_CONFIG_KEYS = ['kind', 'directory']
     """Common source config keys
@@ -293,10 +312,10 @@ class Source(Plugin):
         Args:
            ref (simple object): The internal source reference to set, or ``None``
            node (dict): The same dictionary which was previously passed
-                        to :func:`~buildstream.source.Source.configure`
+                        to :func:`Plugin.configure() <buildstream.plugin.Plugin.configure>`
 
-        See :func:`~buildstream.source.Source.get_ref` for a discussion on
-        the *ref* parameter.
+        See :func:`Source.get_ref() <buildstream.source.Source.get_ref>`
+        for a discussion on the *ref* parameter.
 
         .. note::
 
@@ -320,8 +339,8 @@ class Source(Plugin):
         backend store allows one to query for a new ref from a symbolic
         tracking data without downloading then that is desirable.
 
-        See :func:`~buildstream.source.Source.get_ref` for a discussion on
-        the *ref* parameter.
+        See :func:`Source.get_ref() <buildstream.source.Source.get_ref>`
+        for a discussion on the *ref* parameter.
         """
         # Allow a non implementation
         return None
@@ -365,7 +384,7 @@ class Source(Plugin):
            :class:`.SourceError`
 
         Default implementation is to call
-        :func:`~buildstream.source.Source.stage`.
+        :func:`Source.stage() <buildstream.source.Source.stage>`.
 
         Implementors overriding this method should assume that *directory*
         already exists.
@@ -375,20 +394,6 @@ class Source(Plugin):
         """
         self.stage(directory)
 
-    def mark_download_url(self, url):
-        """Identifies the URL that this Source uses to download
-
-        This must be called during :func:`~buildstream.plugin.Plugin.configure` if
-        :func:`~buildstream.source.Source.translate_url` is not called.
-
-        Args:
-           url (str): The url used to download
-
-        *Since: 1.2*
-        """
-        alias, _ = url.split(utils._ALIAS_SEPARATOR, 1)
-        self.__expected_alias = alias
-
     def get_source_fetchers(self):
         """Get the objects that are used for fetching
 
@@ -397,8 +402,15 @@ class Source(Plugin):
         is recommended.
 
         Returns:
-           list: A list of SourceFetchers. If SourceFetchers are not supported,
-                 this will be an empty list.
+           iterable: The Source's SourceFetchers, if any.
+
+        .. note::
+
+           Implementors can implement this as a generator.
+
+           The :func:`SourceFetcher.fetch() <buildstream.source.SourceFetcher.fetch>`
+           method will be called on the returned fetchers one by one,
+           before consuming the next fetcher in the list.
 
         *Since: 1.2*
         """
@@ -421,17 +433,27 @@ class Source(Plugin):
         os.makedirs(directory, exist_ok=True)
         return directory
 
-    def translate_url(self, url, *, alias_override=None):
+    def translate_url(self, url, *, alias_override=None, primary=True):
         """Translates the given url which may be specified with an alias
         into a fully qualified url.
 
         Args:
-           url (str): A url, which may be using an alias
+           url (str): A URL, which may be using an alias
            alias_override (str): Optionally, an URI to override the alias with. (*Since: 1.2*)
+           primary (bool): Whether this is the primary URL for the source. (*Since: 1.2*)
 
         Returns:
-           str: The fully qualified url, with aliases resolved
+           str: The fully qualified URL, with aliases resolved
+        .. note::
+
+           This must be called for every URL in the configuration during
+           :func:`Plugin.configure() <buildstream.plugin.Plugin.configure>` if
+           :func:`Source.mark_download_url() <buildstream.source.Source.mark_download_url>`
+           is not called.
         """
+        # Ensure that the download URL is also marked
+        self.mark_download_url(url, primary=primary)
+
         # Alias overriding can happen explicitly (by command-line) or
         # implicitly (the Source being constructed with an __alias_override).
         if alias_override or self.__alias_override:
@@ -450,14 +472,56 @@ class Source(Plugin):
                         url = override_url + url_body
             return url
         else:
-            # Sneakily store the alias if it hasn't already been stored
-            if not self.__expected_alias and url and utils._ALIAS_SEPARATOR in url:
-                url_alias, _ = url.split(utils._ALIAS_SEPARATOR, 1)
-                self.__expected_alias = url_alias
-
             project = self._get_project()
             return project.translate_url(url, first_pass=self.__first_pass)
 
+    def mark_download_url(self, url, *, primary=True):
+        """Identifies the URL that this Source uses to download
+
+        Args:
+           url (str): The URL used to download
+           primary (bool): Whether this is the primary URL for the source
+
+        .. note::
+
+           This must be called for every URL in the configuration during
+           :func:`Plugin.configure() <buildstream.plugin.Plugin.configure>` if
+           :func:`Source.translate_url() <buildstream.source.Source.translate_url>`
+           is not called.
+
+        *Since: 1.2*
+        """
+        # Only mark the Source level aliases on the main instance, not in
+        # a reinstantiated instance in mirroring.
+        if not self.__alias_override:
+            if primary:
+                expected_alias = _extract_alias(url)
+
+                assert (self.__expected_alias is None or
+                        self.__expected_alias == expected_alias), \
+                    "Primary URL marked twice with different URLs"
+
+                self.__expected_alias = expected_alias
+
+        # Enforce proper behaviour of plugins by ensuring that all
+        # aliased URLs have been marked at Plugin.configure() time.
+        #
+        if self._get_configuring():
+            # Record marked urls while configuring
+            #
+            self.__marked_urls.add(url)
+        else:
+            # If an unknown aliased URL is seen after configuring,
+            # this is an error.
+            #
+            # It is still possible that a URL that was not mentioned
+            # in the element configuration can be marked, this is
+            # the case for git submodules which might be automatically
+            # discovered.
+            #
+            assert (url in self.__marked_urls or not _extract_alias(url)), \
+                "URL was not seen at configure time: {}".format(url)
+
     def get_project_directory(self):
         """Fetch the project base directory
 
@@ -522,10 +586,12 @@ class Source(Plugin):
     def _fetch(self):
         project = self._get_project()
         source_fetchers = self.get_source_fetchers()
+
+        # Use the source fetchers if they are provided
+        #
         if source_fetchers:
             for fetcher in source_fetchers:
                 alias = fetcher._get_alias()
-                success = False
                 for uri in project.get_alias_uris(alias, first_pass=self.__first_pass):
                     try:
                         fetcher.fetch(uri)
@@ -534,10 +600,16 @@ class Source(Plugin):
                     except BstError as e:
                         last_error = e
                         continue
-                    success = True
+
+                    # No error, we're done with this fetcher
                     break
-                if not success:
+
+                else:
+                    # No break occurred, raise the last detected error
                     raise last_error
+
+        # Default codepath is to reinstantiate the Source
+        #
         else:
             alias = self._get_alias()
             if self.__first_pass:
@@ -561,7 +633,11 @@ class Source(Plugin):
                 except BstError as e:
                     last_error = e
                     continue
+
+                # No error, we're done here
                 return
+
+            # Re raise the last detected error
             raise last_error
 
     # Wrapper for stage() api which gives the source
@@ -732,7 +808,8 @@ class Source(Plugin):
         #
         # Step 2 - Set the ref in memory, and determine changed state
         #
-        changed = self._set_ref(new_ref, node)
+        if not self._set_ref(new_ref, node):
+            return False
 
         def do_save_refs(refs):
             try:
@@ -769,7 +846,7 @@ class Source(Plugin):
                                   .format(provenance.filename.shortname),
                                   reason="tracking-junction-fragment")
 
-        return changed
+        return True
 
     # Wrapper for track()
     #
@@ -804,12 +881,12 @@ class Source(Plugin):
     # Tries to call track for every mirror, stopping once it succeeds
     def __do_track(self):
         project = self._get_project()
-        # If there are no mirrors, or no aliases to replace, there's nothing to do here.
         alias = self._get_alias()
         if self.__first_pass:
             mirrors = project.first_pass_config.mirrors
         else:
             mirrors = project.config.mirrors
+        # If there are no mirrors, or no aliases to replace, there's nothing to do here.
         if not mirrors or not alias:
             return self.track()
 
@@ -867,3 +944,11 @@ class Source(Plugin):
         _yaml.node_final_assertions(config)
 
         return config
+
+
+def _extract_alias(url):
+    parts = url.split(utils._ALIAS_SEPARATOR, 1)
+    if len(parts) > 1 and not parts[0].lower() in utils._URI_SCHEMES:
+        return parts[0]
+    else:
+        return ""
